#include <RcppArmadillo.h>
// [[Rcpp::depends(RcppArmadillo)]]

using namespace Rcpp;
using namespace arma;

// gaussian kernel 

// [[Rcpp::export]]
mat Kernel(mat x){
  mat res(1,1,fill::zeros);
  res=exp(-pow(x,2)/2);
  return(res);
}

// local linear estimator at each one dimensional X for a given beta
// Arguments:
// h = bandwidth
// X = covariate matrix
// Y = response in the single index model
// beta = pre-given parameter  
// tarX = the support of eta function
// Output:
// A matrix with two rows, where first row is the estimate of eta, 
// and the second row is the estimate of the derivative of eta.

// [[Rcpp::export]]
mat SiMLL(double h, mat X, mat Y, mat beta, mat tarX){
  int M=tarX.n_rows;
  int N=X.n_rows;
  X=X*beta;
  mat res(2,M,fill::zeros);
  for(int k=0; k<M; k++){
    double t=tarX(k,0);
    mat Xt(N,1,fill::ones);
    Xt=X-Xt*t;
    mat Wt(N,1,fill::zeros);
    Wt=Kernel(Xt/h);
    double a=as_scalar(sum(Wt,0));
    double b=as_scalar(sum(Xt%Wt,0));
    double d=as_scalar(sum(Xt%Xt%Wt,0));
    double e=as_scalar(sum(Wt%Y,0));
    double f=as_scalar(sum(Wt%Xt%Y,0));
    double g=a*d-b*b;
    res(0,k)=(d*e-b*f)/g;
    res(1,k)=(a*f-b*e)/g;
  }
  return(res);
}

// delete one row function 

// [[Rcpp::export]]
mat deleRow(mat x, int k1, int k2){
  x.shed_rows(k1-1,k2-1);
  return(x);
}

// Leave one group out local linear estimator at each one dimensional X for a given beta.
// Arguments:
// h = bandwidth
// X = covariate matrix
// Y = response in the single index model
// beta = pre-given parameter  
// ind1 = the starting index in the group for each individual
// ind2 = the ending index in the group for each individual
// Output:
// A matrix with two rows, where first row is the estimate of eta, 
// and the second row is the estimate of the derivative of eta.

// [[Rcpp::export]]
mat SiMLLCV(double h, mat X, mat Y, mat beta, IntegerVector ind1, IntegerVector ind2){
  int N=X.n_rows;
  X=X*beta;
  mat res(2,N,fill::zeros);
  for(int k=0; k<N; k++){
    double t=X(k,0);
    mat Xt(N-ind2(k)+ind1(k)-1,1,fill::ones);
    Xt=deleRow(X,ind1(k),ind2(k))-Xt*t;
    mat Wt=Kernel(Xt/h);
    mat Yt=deleRow(Y,ind1(k),ind2(k));
    double a=as_scalar(sum(Wt,0));
    double b=as_scalar(sum(Xt%Wt,0));
    double d=as_scalar(sum(Xt%Xt%Wt,0));
    double e=as_scalar(sum(Wt%Yt,0));
    double f=as_scalar(sum(Wt%Xt%Yt,0));
    double g=a*d-b*b;
    res(0,k)=(d*e-b*f)/g;
    res(1,k)=(a*f-b*e)/g;
  }
  return(res);
}
